@@ -394,7 +394,7 @@ class Image:
 
 class Human:
     '''人的ai数据处理'''
-    # 17 个 人体关键点对应关系
+    # 17 个 人体关键点对应关系 旧版ai识别逻辑
     HUMAN_BODY_KEY_POINTS = [
         # face
         "nose",
@@ -410,7 +410,7 @@ class Human:
         "left_knee", "right_knee",
         "left_ankle", "right_ankle"
     ]
-    # 裁剪点 映射 ai 识别的 人体关键点
+    # 裁剪点 映射 ai 识别的 人体关键点  旧版ai识别逻辑
     mapping = {
         'eye': ["left_eye", "right_eye"],
         "nose": ["nose", "nose"],
@@ -426,9 +426,9 @@ class Human:
     def __init__(self, image_analysis):
         self.threshold = 2.5  # 置信度 2.5
         self.image_analysis = image_analysis
-        if image_analysis.get('recognition'):
+        if image_analysis.get('recognition'):  # 新版本ai识别结果
             self.threshold = 0.05
-            image_analysis['human_pose'] = self.add_waist_crop_point(image_analysis.get("recognition"),
+            image_analysis['human_pose'] = self.add_new_crop_point(image_analysis.get("recognition"),
                                                                      image_analysis.get("human_pose"))
         self.image_width = None
         self.image_height = None
@@ -453,11 +453,48 @@ class Human:
             }
         return ai_crop_point_dict
 
-    def add_waist_crop_point(self, recognition, human_pose):
+    @staticmethod
+    def calculate_mandible_crop_point(face):
         """
-        添加腰部识别点
+        在recognition face 内
+        置信度小于0.8不可信
+        嘴巴点 在 keypoints index 3 4
+        下巴点 在 bbox index  2 3
+        :return: 下颚 的点
+        """
+        mandible_point = [-1, -1 ,0]
+        # face 不存在识别结果
+        if face is None or not face:
+            return mandible_point
+
+        # 置信度小于0.8不可信
+        face_confidece = face['confidence']
+        if face_confidece < 0.8:
+            return mandible_point
+
+        mandible_point[2] = face['confidence']
+        # 下巴y点
+        law_y = face['bbox'][3]  # 下巴纵轴
+        left_mouth = face['landmarks'][3]  # 嘴的左边
+        right_mouth = face['landmarks'][4]  # 嘴的右边
+        # 嘴巴的中点
+        mouth_x = min(left_mouth[0], right_mouth[0]) + abs(left_mouth[0] - right_mouth[0])/2
+        mouth_y = max(left_mouth[1], right_mouth[1])
+        # 下颚的点
+        mandible_point[0] = mouth_x
+        mandible_point[1] = min(law_y, mouth_y) + abs(mouth_y - law_y)/2
+        # print(mandible_point)
+        return mandible_point
+
+    def add_new_crop_point(self, recognition, human_pose):
+        """
+        把 recognition 内的数据 添加到 旧版的17个人体关键点的数据结构中
+
+        添加识别点
+            腰部：waist
+            下颚：mandible
         recognition 数据解释
-        https://www.teambition.com/project/5e0dba42b2bba90021d9a872/app/5eba5fba6a92214d420a3219/workspaces/5fe40d962aa6d50046cc2bff/docs/60c9a039eaa11900013dc6cf
+        https://www.teambition.com/project/5e0dba42b2bba90021d9a872/app/5eba5fba6a92214d420a3219/workspaces/5fe40d962aa6d50046cc2bff/docs/60c1764beaa11900011f146e
         """
         if (human_pose is None) or (not human_pose):
             return human_pose
@@ -473,7 +510,8 @@ class Human:
         if len(human_bodies_list) != len(human_pose_result):
             return human_pose
 
-        # 18 个 人体关键点对应关系
+        # 添加完裁剪点后，下面两个配置需要对应的改掉
+        # 19 个 人体关键点对应关系
         self.HUMAN_BODY_KEY_POINTS = [
             # face
             "nose",
@@ -481,6 +519,7 @@ class Human:
             "right_eye",
             "left_ear",
             "right_ear",
+            "mandible",
             # upper body
             "left_shoulder",
             "right_shoulder",
@@ -503,6 +542,7 @@ class Human:
             "eye": ["left_eye", "right_eye"],
             "nose": ["nose", "nose"],
             "ear": ["left_ear", "right_ear"],
+            "mandible": ["mandible", "mandible"],
             "shoulder": ["left_shoulder", "right_shoulder"],
             "elbow": ["left_elbow", "right_elbow"],
             "wrist": ["left_wrist", "right_wrist"],
@@ -513,10 +553,18 @@ class Human:
         }
 
         for i, human in enumerate(human_bodies_list):
-            # index 为6 的是腰部
             if len(human_pose_result[i]['keypoints']) == 17:
+                # 在recognition keypoints 内 index 为6 的是腰部
                 human_pose_result[i]['keypoints'].insert(self.HUMAN_BODY_KEY_POINTS.index('waist'),
                                                          human['keypoints'][6])
+
+                # 在recognition face 内
+                # 置信度小于0.8不可信
+                # 嘴巴点 在 keypoints index 3 4
+                # 下巴点 在 bbox index  2 3
+                human_pose_result[i]['keypoints'].insert(self.HUMAN_BODY_KEY_POINTS.index('mandible'),
+                                                         self.calculate_mandible_crop_point(human['face']))
+
         human_pose_dict['result'] = human_pose_result
 
         human_pose = json.dumps(human_pose_dict)
@@ -525,6 +573,8 @@ class Human:
         #     logging.error(f"ai识别结果-人体关键点-解析失败: {self.image_analysis} ", exc_info=True)
         return human_pose
 
+
+
     def deal_human_crop_point(self, crop_point_dict):
         '''
         处理人体的裁剪点，方便裁剪时取值
